@@ -142,62 +142,85 @@ setMethod("t", signature(x = "dgeMatrix"),
               x })
 
 setMethod("crossprod", signature(x = "dgeMatrix", y = "missing"),
-          function(x, y = NULL)
-          .Call("dgeMatrix_crossprod", x),
+          function(x, y = NULL) .Call("dgeMatrix_crossprod", x),
           valueClass = "dpoMatrix")
 
 setMethod("crossprod", signature(x = "dgeMatrix", y = "dgeMatrix"),
-          function(x, y = NULL)
-          .Call("dgeMatrix_dgeMatrix_crossprod", x, y),
+          function(x, y = NULL) .Call("dgeMatrix_dgeMatrix_crossprod", x, y),
           valueClass = "dgeMatrix")
 
 setMethod("crossprod", signature(x = "dgeMatrix", y = "matrix"),
-          function(x, y = NULL)
-          .Call("dgeMatrix_matrix_crossprod", x, y),
+          function(x, y = NULL) .Call("dgeMatrix_matrix_crossprod", x, y),
           valueClass = "dgeMatrix")
-
 setMethod("crossprod", signature(x = "dgeMatrix", y = "numeric"),
           function(x, y = NULL)
           .Call("dgeMatrix_matrix_crossprod", x, as.matrix(y)),
           valueClass = "dgeMatrix")
+setMethod("crossprod", signature(x = "matrix", y = "dgeMatrix"),
+          function(x, y = NULL) callGeneric(as(x, "dgeMatrix"), y),
+          valueClass = "dgeMatrix")
+setMethod("crossprod", signature(x = "numeric", y = "dgeMatrix"),
+          function(x, y = NULL) callGeneric(as.matrix(x), y),
+          valueClass = "dgeMatrix")
+
+setMethod("%*%", signature(x = "dgeMatrix", y = "dgeMatrix"),
+          function(x, y) .Call("dgeMatrix_matrix_mm", x, y, TRUE, FALSE),
+          valueClass = "dgeMatrix")
+
+## dgeMatrix <-> matrix ("matrix" dispatches before "numeric" since R 2.1.0)
+setMethod("%*%", signature(x = "dgeMatrix", y = "matrix"),
+          function(x, y) .Call("dgeMatrix_matrix_mm", x, y, FALSE, FALSE),
+          valueClass = "dgeMatrix")
+
+
+setMethod("%*%", signature(x = "matrix", y = "dgeMatrix"),
+          function(x, y) .Call("dgeMatrix_matrix_mm", y, x, FALSE, TRUE),
+          valueClass = "dgeMatrix")
+
+## dgeMatrix <-> numeric : uses the "matrix" one
+setMethod("%*%", signature(x = "dgeMatrix", y = "numeric"),
+          function(x, y)
+          .Call("dgeMatrix_matrix_mm", x, as.matrix(y), FALSE, FALSE),
+          valueClass = "dgeMatrix")
+
+setMethod("%*%", signature(x = "numeric", y = "dgeMatrix"),
+          function(x, y)
+          .Call("dgeMatrix_matrix_mm", y, rbind(x), FALSE, TRUE),
+          valueClass = "dgeMatrix")
 
 setMethod("diag", signature(x = "dgeMatrix"),
           function(x = 1, nrow, ncol = n)
           .Call("dgeMatrix_getDiag", x))
 
 ## should be done once and for all in ./Matrix.R - but that fails (as "show"):
-setMethod("dim", signature(x = "dgeMatrix"),
-          function(x) x@Dim, valueClass = "integer")
-setMethod("dimnames", signature(x = "dgeMatrix"), function(x) x@Dimnames)
-
-## not exported but used more than once for "dimnames<-" method :
-## -- or do only once for all "Matrix" classes ??
-dimnamesGets <- function (x, value) {
-    d <- dim(x)
-    if (!is.list(value) || length(value) != 2 ||
-        !(is.null(v1 <- value[[1]]) || length(v1) == d[1]) ||
-        !(is.null(v2 <- value[[2]]) || length(v2) == d[2]))
-        stop(sprintf("invalid dimnames given for '%s' object", class(x)))
-    x@Dimnames <- list(if(!is.null(v1)) as.character(v1),
-                       if(!is.null(v2)) as.character(v2))
-    x
-}
-setMethod("dimnames<-", signature(x = "dgeMatrix", value = "list"),
-          dimnamesGets)
+## setMethod("dim", signature(x = "dgeMatrix"),
+##           function(x) x@Dim, valueClass = "integer")
+## setMethod("dimnames", signature(x = "dgeMatrix"), function(x) x@Dimnames)
 
+## setMethod("dimnames<-", signature(x = "dgeMatrix", value = "list"),
+##           dimnamesGets)
 
+## DB - I don't think this is a good idea without first checking symmetry
+#setMethod("chol", signature(x = "dgeMatrix", pivot = "ANY"), cholMat)
 
 setMethod("solve", signature(a = "dgeMatrix", b = "missing"),
-          function(a, b, ...) .Call("dgeMatrix_solve", a)
-          )
+          function(a, b, ...) .Call("dgeMatrix_solve", a),
+          valueClass = "dgeMatrix")
 
 setMethod("solve", signature(a = "dgeMatrix", b = "matrix"),
-          function(a, b, ...)
-          .Call("dgeMatrix_matrix_solve", a, b)
-          )
+          function(a, b, ...) .Call("dgeMatrix_matrix_solve", a, b, FALSE),
+          valueClass = "dgeMatrix")
+
+setMethod("solve", signature(a = "dgeMatrix", b = "dgeMatrix"),
+          function(a, b, ...) .Call("dgeMatrix_matrix_solve", a, b, TRUE),
+          valueClass = "dgeMatrix")
+
+setMethod("solve", signature(a = "dgeMatrix", b = "numeric"),
+          function(a, b, ...) .Call("dgeMatrix_matrix_solve", a, as.matrix(b), TRUE),
+          valueClass = "dgeMatrix")
 
 setMethod("lu", signature(x = "dgeMatrix"),
-          function(x, ...) .Call("dgeMatrix_LU", x))
+          function(x, ...) .Call("dgeMatrix_LU", x), valueClass = "LU")
 
 setMethod("determinant", signature(x = "dgeMatrix", logarithm = "missing"),
           function(x, logarithm, ...) determinant(x, TRUE))
@@ -206,9 +229,6 @@ setMethod("determinant", signature(x = "dgeMatrix", logarithm = "logical"),
           function(x, logarithm, ...)
           .Call("dgeMatrix_determinant", x, logarithm))
 
-setMethod("%*%", signature(x = "dgeMatrix", y = "dgeMatrix"),
-          function(x, y)
-          .Call("dgeMatrix_dgeMatrix_mm", x, y))
 
 setMethod("expm", signature(x = "dgeMatrix"),
           function(x) .Call("dgeMatrix_exp", x),
