@@ -1,3 +1,8 @@
+#### Positive-definite Symmetric Matrices -- Coercion and Methods
+
+setAs("dpoMatrix", "dppMatrix",
+      function(from) as(as(from, "dspMatrix"), "dppMatrix"))
+
 setMethod("chol", signature(x = "dpoMatrix"),
           function(x, pivot, LINPACK)
           .Call("dpoMatrix_chol", x))
