@@ -1,14 +1,32 @@
+#### Symmetric Sparse Matrices in compressed column-oriented format
+
+### contains = "dgCMatrix"
+
+setAs("dsCMatrix", "dgTMatrix",
+      function(from) .Call("dsCMatrix_to_dgTMatrix", from))
+
+setAs("dsCMatrix", "dgeMatrix",
+      function(from) as(as(from, "dgTMatrix"), "dgeMatrix"))
+setAs("dsCMatrix", "matrix",
+      function(from) as(as(from, "dgTMatrix"), "matrix"))
+
+if(FALSE) { ## << FIXME
+setAs("dsCMatrix", "dsTMatrix",
+      function(from) { }) ## < follow "csc_to_dgTMatrix" in ../src/dgTMatrix.c
+setAs("dsCMatrix", "dsyMatrix",
+      function(from) as(as(from, "dsTMatrix"), "dsyMatrix"))
+}# not yet
+
+
 setMethod("solve", signature(a = "dsCMatrix", b = "matrix"),
           function(a, b, ...)
           .Call("dsCMatrix_matrix_solve", a, b))
 
 setMethod("chol", signature(x = "dsCMatrix", pivot = "missing"),
-          function(x, pivot, LINPACK)
-          .Call("dsCMatrix_chol", x, TRUE))
+          function(x, pivot, LINPACK) .Call("dsCMatrix_chol", x, TRUE))
 
 setMethod("chol", signature(x = "dsCMatrix", pivot = "logical"),
-          function(x, pivot, LINPACK)
-          .Call("dsCMatrix_chol", x, pivot))
+          function(x, pivot, LINPACK) .Call("dsCMatrix_chol", x, pivot))
 
 setMethod("t", signature(x = "dsCMatrix"),
           function(x) .Call("ssc_transpose", x),
@@ -27,13 +45,3 @@ setMethod("determinant", signature(x = "dsCMatrix", logarithm = "logical"),
           class(val) <- "det"
           val
       })
-
-setAs("dsCMatrix", "dgTMatrix",
-      function(from) .Call("dsCMatrix_to_dgTMatrix", from))
-
-setAs("dsCMatrix", "dgeMatrix",
-      function(from) as(as(from, "dgTMatrix"), "dgeMatrix"))
-
-setAs("dsCMatrix", "matrix",
-      function(from) as(as(from, "dgTMatrix"), "matrix"))
-
