@@ -1,7 +1,7 @@
 setMethod("eigen", signature(x = "dgeMatrix", only.values = "missing"),
-          function(x, symmetric, only.values, EISPACK) {
-              nCall = match.call()
-              nCall$only.values = FALSE
+          function(x, symmetric, only.values, EISPACK) {# << must match generic
+              nCall <- match.call()
+              nCall$only.values <- FALSE
               eval(nCall, parent.frame())
           })
 
@@ -12,7 +12,7 @@ setMethod("eigen", signature(x = "dgeMatrix", only.values = "logical"),
 
 setMethod("Schur", signature(x = "dgeMatrix", vectors = "missing"),
           function(x, vectors, ...) Schur(x, TRUE, ...))
-         
+
 setMethod("Schur", signature(x = "dgeMatrix", vectors = "logical"),
           function(x, vectors, ...) .Call("dgeMatrix_Schur", x, vectors)
           )
