@@ -7,9 +7,10 @@
 #include "triplet_to_col.h"
 
 SEXP csc_crossprod(SEXP x);
+SEXP csc_tcrossprod(SEXP x);
 SEXP csc_matrix_crossprod(SEXP x, SEXP y);
 SEXP dgCMatrix_validate(SEXP x);
-SEXP csc_to_dgTMatrix(SEXP x);
+SEXP compressed_to_dgTMatrix(SEXP x, SEXP colP);
 SEXP csc_to_matrix(SEXP x);
 SEXP csc_to_dgeMatrix(SEXP x);
 SEXP matrix_to_csc(SEXP A);
